#ifndef XICORE_H
#define XICORE_H

#include <xiCoreGlobal.h>
#include <xiCoreLog.h>
#include <xiCoreVersion.h>
#include <xiCrash.h>
#include <xiRtti.h>

#include <Chainable.h>
#include <PicBuf.h>
#include <PicBufMacros.h>
#include <PicBufAPI.h>
#include <ValueTypes.h>
#include <CameraParameters.h>
#include <CameraManager.h>
#include <Data.h>
#include <Chainable.h>
#include <Chain.h>
#include <ChainManager.h>
#include <MemoryManager.h>
#include <ImageFormatManager.h>
#include <AppDelegate.h>
#include <DataSaver.h>
#include <DataLoader.h>
#include <ImageData.h>
#include <ImageFormat.h>
#include <ImageMetadata.h>
#include <Plugin.h>
#include <VideoDecoder.h>
#include <VideoDecoder.h>
#include <VideoSource.h>
#include <ViewImageReceiver.h>

#endif // XICORE_H
