'''
data types as declared in library ctypes.wintypes in Python for windows
'''
from ctypes import c_uint, c_float, c_void_p, c_double

DWORD = c_uint
LPVOID = c_void_p
HANDLE = c_void_p
FLOAT = c_float
DOUBLE = c_double
